# coding: utf-8

"""
Binance Derivatives Trading Portfolio Margin Pro WebSocket Market Streams

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin Pro WebSocket Market Streams
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""


from __future__ import annotations
import json
import pprint
from pydantic import (
    BaseModel,
    ConfigDict,
    ValidationError,
    field_validator,
)
from typing import Any, Optional
from binance_sdk_derivatives_trading_portfolio_margin_pro.websocket_streams.models.risklevelchange import (
    Risklevelchange,
)
from typing import Union, Set, Dict
from typing_extensions import Self

USERDATASTREAMEVENTSRESPONSE_ONE_OF_SCHEMAS = ["Risklevelchange"]


class UserDataStreamEventsResponse(BaseModel):
    """
    UserDataStreamEventsResponse
    """

    # data type: Risklevelchange
    oneof_schema_1_validator: Optional[Risklevelchange] = None
    actual_instance: Optional[Union[Risklevelchange]] = None
    one_of_schemas: Set[str] = {"Risklevelchange"}

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {}

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError(
                    "If a position argument is used, only 1 is allowed to set `actual_instance`"
                )
            if kwargs:
                raise ValueError(
                    "If a position argument is used, keyword arguments cannot be used."
                )
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator("actual_instance")
    def actual_instance_must_validate_oneof(cls, v):

        error_messages = []
        match = 0
        # validate data type: Risklevelchange
        if not isinstance(v, Risklevelchange):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Risklevelchange`"
            )
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError(
                "Multiple matches found when setting `actual_instance` in UserDataStreamEventsResponse with oneOf schemas: Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        elif match == 0:
            # no match
            raise ValueError(
                "No match found when setting `actual_instance` in UserDataStreamEventsResponse with oneOf schemas: Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into Risklevelchange
        try:
            instance.actual_instance = Risklevelchange.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError(
                "Multiple matches found when deserializing the JSON string into UserDataStreamEventsResponse with oneOf schemas: Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        elif match == 0:
            # no match
            raise ValueError(
                "No match found when deserializing the JSON string into UserDataStreamEventsResponse with oneOf schemas: Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(
            self.actual_instance.to_json
        ):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], Risklevelchange]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(
            self.actual_instance.to_dict
        ):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())



"""
Binance Derivatives Trading Portfolio Margin Pro REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin Pro REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from typing import Optional
from requests import Session
from binance_common.configuration import ConfigurationRestAPI
from binance_common.models import ApiResponse
from binance_common.signature import Signers
from binance_common.utils import send_request

from ..models import GetPortfolioMarginAssetLeverageResponse
from ..models import PortfolioMarginCollateralRateResponse
from ..models import PortfolioMarginProTieredCollateralRateResponse
from ..models import QueryPortfolioMarginAssetIndexPriceResponse


class MarketDataApi:
    """API Client for MarketDataApi endpoints."""

    def __init__(
        self,
        configuration: ConfigurationRestAPI = None,
        session: Session = None,
        signer: Signers = None,
    ) -> None:
        self._configuration = configuration
        self._session = session
        self._signer = signer

    def get_portfolio_margin_asset_leverage(
        self,
    ) -> ApiResponse[GetPortfolioMarginAssetLeverageResponse]:
        """
                Get Portfolio Margin Asset Leverage(USER_DATA)
                GET /sapi/v1/portfolio/margin-asset-leverage
                https://developers.binance.com/docs/derivatives/portfolio-margin-pro/market-data/Get-Portfolio-Margin-Asset-Leverage

                Get Portfolio Margin Asset Leverage

        Weight: 50

                Args:

                Returns:
                    ApiResponse[GetPortfolioMarginAssetLeverageResponse]

                Raises:
                    RequiredError: If a required parameter is missing.

        """

        payload = None

        return send_request(
            self._session,
            self._configuration,
            method="GET",
            path="/sapi/v1/portfolio/margin-asset-leverage",
            payload=payload,
            time_unit=self._configuration.time_unit,
            response_model=GetPortfolioMarginAssetLeverageResponse,
            is_signed=True,
            signer=self._signer,
        )

    def portfolio_margin_collateral_rate(
        self,
    ) -> ApiResponse[PortfolioMarginCollateralRateResponse]:
        """
                Portfolio Margin Collateral Rate(MARKET_DATA)
                GET /sapi/v1/portfolio/collateralRate
                https://developers.binance.com/docs/derivatives/portfolio-margin-pro/market-data/Classic-Portfolio-Margin-Collateral-Rate

                Portfolio Margin Collateral Rate

        Weight: 50

                Args:

                Returns:
                    ApiResponse[PortfolioMarginCollateralRateResponse]

                Raises:
                    RequiredError: If a required parameter is missing.

        """

        payload = None

        return send_request(
            self._session,
            self._configuration,
            method="GET",
            path="/sapi/v1/portfolio/collateralRate",
            payload=payload,
            time_unit=self._configuration.time_unit,
            response_model=PortfolioMarginCollateralRateResponse,
        )

    def portfolio_margin_pro_tiered_collateral_rate(
        self,
        recv_window: Optional[int] = None,
    ) -> ApiResponse[PortfolioMarginProTieredCollateralRateResponse]:
        """
                Portfolio Margin Pro Tiered Collateral Rate(USER_DATA)
                GET /sapi/v2/portfolio/collateralRate
                https://developers.binance.com/docs/derivatives/portfolio-margin-pro/market-data/Portfolio-Margin-Pro-Tiered-Collateral-Rate

                Portfolio Margin PRO Tiered Collateral Rate

        Weight: 50

                Args:
                    recv_window (Optional[int] = None):

                Returns:
                    ApiResponse[PortfolioMarginProTieredCollateralRateResponse]

                Raises:
                    RequiredError: If a required parameter is missing.

        """

        payload = {"recv_window": recv_window}

        return send_request(
            self._session,
            self._configuration,
            method="GET",
            path="/sapi/v2/portfolio/collateralRate",
            payload=payload,
            time_unit=self._configuration.time_unit,
            response_model=PortfolioMarginProTieredCollateralRateResponse,
            is_signed=True,
            signer=self._signer,
        )

    def query_portfolio_margin_asset_index_price(
        self,
        asset: Optional[str] = None,
    ) -> ApiResponse[QueryPortfolioMarginAssetIndexPriceResponse]:
        """
                Query Portfolio Margin Asset Index Price (MARKET_DATA)
                GET /sapi/v1/portfolio/asset-index-price
                https://developers.binance.com/docs/derivatives/portfolio-margin-pro/market-data/Query-Portfolio-Margin-Asset-Index-Price

                Query Portfolio Margin Asset Index Price

        Weight: 1 if send asset or 50 if not send asset

                Args:
                    asset (Optional[str] = None):

                Returns:
                    ApiResponse[QueryPortfolioMarginAssetIndexPriceResponse]

                Raises:
                    RequiredError: If a required parameter is missing.

        """

        payload = {"asset": asset}

        return send_request(
            self._session,
            self._configuration,
            method="GET",
            path="/sapi/v1/portfolio/asset-index-price",
            payload=payload,
            time_unit=self._configuration.time_unit,
            response_model=QueryPortfolioMarginAssetIndexPriceResponse,
        )



"""
Binance Derivatives Trading Portfolio Margin Pro WebSocket Market Streams

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin Pro WebSocket Market Streams
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from .risklevelchange import Risklevelchange as Risklevelchange
from .user_data_stream_events_response import (
    UserDataStreamEventsResponse as UserDataStreamEventsResponse,
)



"""
Binance Derivatives Trading Portfolio Margin Pro REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin Pro REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from .bnb_transfer_response import BnbTransferResponse as BnbTransferResponse
from .change_auto_repay_futures_status_response import (
    ChangeAutoRepayFuturesStatusResponse as ChangeAutoRepayFuturesStatusResponse,
)
from .fund_auto_collection_response import (
    FundAutoCollectionResponse as FundAutoCollectionResponse,
)
from .fund_collection_by_asset_response import (
    FundCollectionByAssetResponse as FundCollectionByAssetResponse,
)
from .get_auto_repay_futures_status_response import (
    GetAutoRepayFuturesStatusResponse as GetAutoRepayFuturesStatusResponse,
)
from .get_portfolio_margin_asset_leverage_response import (
    GetPortfolioMarginAssetLeverageResponse as GetPortfolioMarginAssetLeverageResponse,
)
from .get_portfolio_margin_asset_leverage_response_inner import (
    GetPortfolioMarginAssetLeverageResponseInner as GetPortfolioMarginAssetLeverageResponseInner,
)
from .get_portfolio_margin_pro_account_balance_response import (
    GetPortfolioMarginProAccountBalanceResponse as GetPortfolioMarginProAccountBalanceResponse,
)
from .get_portfolio_margin_pro_account_balance_response_inner import (
    GetPortfolioMarginProAccountBalanceResponseInner as GetPortfolioMarginProAccountBalanceResponseInner,
)
from .get_portfolio_margin_pro_account_info_response import (
    GetPortfolioMarginProAccountInfoResponse as GetPortfolioMarginProAccountInfoResponse,
)
from .get_portfolio_margin_pro_span_account_info_response import (
    GetPortfolioMarginProSpanAccountInfoResponse as GetPortfolioMarginProSpanAccountInfoResponse,
)
from .get_portfolio_margin_pro_span_account_info_response_risk_unit_mm_list_inner import (
    GetPortfolioMarginProSpanAccountInfoResponseRiskUnitMMListInner as GetPortfolioMarginProSpanAccountInfoResponseRiskUnitMMListInner,
)
from .get_transferable_earn_asset_balance_for_portfolio_margin_response import (
    GetTransferableEarnAssetBalanceForPortfolioMarginResponse as GetTransferableEarnAssetBalanceForPortfolioMarginResponse,
)
from .mint_bfusd_for_portfolio_margin_response import (
    MintBfusdForPortfolioMarginResponse as MintBfusdForPortfolioMarginResponse,
)
from .portfolio_margin_collateral_rate_response import (
    PortfolioMarginCollateralRateResponse as PortfolioMarginCollateralRateResponse,
)
from .portfolio_margin_collateral_rate_response_inner import (
    PortfolioMarginCollateralRateResponseInner as PortfolioMarginCollateralRateResponseInner,
)
from .portfolio_margin_pro_bankruptcy_loan_repay_response import (
    PortfolioMarginProBankruptcyLoanRepayResponse as PortfolioMarginProBankruptcyLoanRepayResponse,
)
from .portfolio_margin_pro_tiered_collateral_rate_response import (
    PortfolioMarginProTieredCollateralRateResponse as PortfolioMarginProTieredCollateralRateResponse,
)
from .portfolio_margin_pro_tiered_collateral_rate_response_inner import (
    PortfolioMarginProTieredCollateralRateResponseInner as PortfolioMarginProTieredCollateralRateResponseInner,
)
from .portfolio_margin_pro_tiered_collateral_rate_response_inner_collateral_info_inner import (
    PortfolioMarginProTieredCollateralRateResponseInnerCollateralInfoInner as PortfolioMarginProTieredCollateralRateResponseInnerCollateralInfoInner,
)
from .query_portfolio_margin_asset_index_price_response import (
    QueryPortfolioMarginAssetIndexPriceResponse as QueryPortfolioMarginAssetIndexPriceResponse,
)
from .query_portfolio_margin_asset_index_price_response_inner import (
    QueryPortfolioMarginAssetIndexPriceResponseInner as QueryPortfolioMarginAssetIndexPriceResponseInner,
)
from .query_portfolio_margin_pro_bankruptcy_loan_amount_response import (
    QueryPortfolioMarginProBankruptcyLoanAmountResponse as QueryPortfolioMarginProBankruptcyLoanAmountResponse,
)
from .query_portfolio_margin_pro_bankruptcy_loan_repay_history_response import (
    QueryPortfolioMarginProBankruptcyLoanRepayHistoryResponse as QueryPortfolioMarginProBankruptcyLoanRepayHistoryResponse,
)
from .query_portfolio_margin_pro_bankruptcy_loan_repay_history_response_rows_inner import (
    QueryPortfolioMarginProBankruptcyLoanRepayHistoryResponseRowsInner as QueryPortfolioMarginProBankruptcyLoanRepayHistoryResponseRowsInner,
)
from .query_portfolio_margin_pro_negative_balance_interest_history_response import (
    QueryPortfolioMarginProNegativeBalanceInterestHistoryResponse as QueryPortfolioMarginProNegativeBalanceInterestHistoryResponse,
)
from .query_portfolio_margin_pro_negative_balance_interest_history_response_inner import (
    QueryPortfolioMarginProNegativeBalanceInterestHistoryResponseInner as QueryPortfolioMarginProNegativeBalanceInterestHistoryResponseInner,
)
from .redeem_bfusd_for_portfolio_margin_response import (
    RedeemBfusdForPortfolioMarginResponse as RedeemBfusdForPortfolioMarginResponse,
)
from .repay_futures_negative_balance_response import (
    RepayFuturesNegativeBalanceResponse as RepayFuturesNegativeBalanceResponse,
)
from .transfer_ldusdt_for_portfolio_margin_response import (
    TransferLdusdtForPortfolioMarginResponse as TransferLdusdtForPortfolioMarginResponse,
)
